#pragma ident "$Id: rt593.c,v 1.6 2011/10/19 16:21:07 dechavez Exp $"
/*======================================================================
 *
 * RT593 and 9/10/11 GPS bug support
 *
 *====================================================================*/
#include "isidl.h"

#define MY_MOD_ID ISIDL_MOD_RT593

#define SUBFORM_OFF 23        /* IDA 9.x subformat */
#define RT593_OFF   55        /* system time delay factor */
#define UNUSED_OFF  58        /* unused bytes */
#define BEGSYS_OFF  10        /* begin system time */
#define ENDSYS_OFF  42        /* end system time */
#define CHAN_OFF    36        /* channel number */
#define REV9_TAG_OFFSET     2 /* offset to IDA9 tag */
#define RT593_FILTER_DELAY 58 /* 290 msec filter delay */
#define BEGEXT_OFF  14        /* external time (begin time if data packet) */
#define ENDEXT_OFF  46        /* end external time (for data packets) */

static void ApplyPatch(UINT8 *payload)
{
UINT32 value, correction = RT593_FILTER_DELAY;

    utilUnpackUINT32(&payload[BEGSYS_OFF], &value);
    utilPackUINT32(&payload[BEGSYS_OFF], value - correction);

    utilUnpackUINT32(&payload[ENDSYS_OFF], &value);
    utilPackUINT32(&payload[ENDSYS_OFF], value - correction);

    payload[SUBFORM_OFF] |= 1;
    payload[RT593_OFF] = (char) correction;
    payload[UNUSED_OFF] = payload[UNUSED_OFF+1] = 0;
}

void RT593PatchTest(ISI_RAW_PACKET *raw)
{
UINT16 chan;
static char *patched = IDA_REV9_TAG_X;

/* Early exit if patched upstream */

    if (memcmp(&raw->payload[REV9_TAG_OFFSET], patched, strlen(patched)) == 0) return;

/* Patch 24-bit channels only */

    utilUnpackUINT16(&raw->payload[CHAN_OFF], &chan);
    if (chan >= 22 && chan <= 24) {
        ApplyPatch(raw->payload);
        memcpy(&raw->payload[REV9_TAG_OFFSET], patched, strlen(patched));
    }
}

static void Patch91011Extime(UINT8 *field, INT32 correction)
{
UINT32 value;

    utilUnpackUINT32(field, &value);
    utilPackUINT32(field, value + correction);
}

void Patch91011Bug(ISI_RAW_PACKET *raw, int daybug)
{
INT32 correction;

    correction = daybug * SEC_PER_DAY;

    Patch91011Extime(&raw->payload[BEGEXT_OFF], correction);
    raw->payload[SUBFORM_OFF] |= IDA9_FLAG_91011_BUG_PATCH;
    if (ida_rtype(raw->payload, 9) == IDA_DATA) Patch91011Extime(&raw->payload[ENDEXT_OFF], correction);
}

/* Revision History
 *
 * $Log: rt593.c,v $
 * Revision 1.6  2011/10/19 16:21:07  dechavez
 * changed static 91011bug to variable daybug to accomodate new behavior in these crazy clocks
 *
 * Revision 1.5  2011/10/14 17:23:16  dechavez
 * added Patch91011Extime()
 * removed uneeded dl argument to RT593PatchTest()
 *
 * Revision 1.4  2011/10/13 18:07:46  dechavez
 * OR in the version number, to avoid clobbering anything that might be in the flags nibble
 *
 * Revision 1.3  2008/03/05 23:23:07  dechavez
 * fixed comment about unused bytes at offset 58
 *
 * Revision 1.2  2007/12/20 23:12:44  dechavez
 * Fixed bug where 16-bit streams in RT593 equipped ARS systems were getting
 * flagged as Ida9 instead of IDA9
 *
 * Revision 1.1  2007/03/26 21:50:40  dechavez
 * initial release
 *
 */
