#pragma ident "$Id: main.c,v 1.1 2013/03/11 23:18:47 dechavez Exp $"
/*======================================================================
 *
 *  Read and parse Iridium SBD messages, assume ADDOSS payloads
 *
 *====================================================================*/
#include "sbd.h"
#include "addoss.h"

static void PrintSBD(SBD_MESSAGE *message)
{
    if (message->mo_header.valid) {
        printf("MO header: ");
        printf("reference no = %lu ", message->mo_header.cdr);
        printf("IMEI = %s ",          message->mo_header.imei);
        printf("status = %d ",        message->mo_header.status);
        printf("momsn = %hd ",        message->mo_header.momsn);
        printf("mtmsn = %hd ",        message->mo_header.mtmsn);
        printf("tstamp = %s\n",       utilLttostr(message->mo_header.tstamp, 0, NULL));
    }

    if (message->mt_header.valid) {
        printf("MT header: ");
        printf("client id = 0x%08x ", message->mt_header.myid);
        printf("IMEI = %s ",          message->mt_header.imei);
        printf("flags = 0x%04x\n",    message->mt_header.flags);
    }
}

static void PrintOSG_STATUS(UINT8 *start)
{
ADDOSS_OSG_STATUS status;

    addossUnpackOSGstatus(start, &status);
    printf("OSG status frame\n");
    printf("----------------\n");
    printf("            reason = ");
    switch (status.reason) {
      case 1:  printf("heartbeat\n"); break;
      case 2:  printf("lost OBS comm\n"); break;
      case 3:  printf("OBS ping received\n"); break;
      default: printf("unknown reason '%d'\n", status.reason);
    }
    printf("            tstamp = %s\n", utilLttostr(status.tstamp.epoch, 0, NULL));
    printf("          nnCAMERR = %d\n", status.nCAMERR);
    printf("       nSendPacket = %d\n", status.nSendPacket);
    printf("nSendPacketRetries = %d\n", status.nSendPacketRetries);
    printf("      nPacketsSent = %d\n", status.nPacketsSent);
    printf("  nPacketsReceived = %d\n", status.nPacketsReceived);
    printf("Last packet received status\n");
    printf("---------------------------\n");
    printf("              mode = %d\n", status.lastpacket.mode);
    printf("          PSKerror = %d\n", status.lastpacket.PSKerror);
    printf("   nFramesExpected = %d\n", status.lastpacket.nFramesExpected);
    printf("        nCRCerrors = %d\n", status.lastpacket.nCRCerrors);
    printf("  MeanSquaredError = %c%d\n", status.lastpacket.msesign  ? '-' : '+', status.lastpacket.MeanSquaredError);
    printf("RecvSignalStrength = %c%d\n", status.lastpacket.rsssign  ? '-' : '+', status.lastpacket.RecvSignalStrength);
    printf("          InputSNR = %c%d\n", status.lastpacket.isnrsign ? '-' : '+', status.lastpacket.InputSNR);
    printf("         OutputSNR = %c%d\n", status.lastpacket.osnrsign ? '-' : '+', status.lastpacket.OutputSNR);
    printf("       StdDevNoise = %c%d\n", status.lastpacket.sdnsign  ? '-' : '+', status.lastpacket.StdDevNoise);
    printf("            tstamp = %s\n", utilLttostr(status.lastpacket.tstamp.epoch, 0, NULL));
}

static void PrintADDOSSframe(SBD_PAYLOAD *payload)
{
UINT8 *ptr;
UINT16 type;

    if (!payload->valid) return;
    printf("%d bytes of user payload\n", payload->len);
    utilPrintHexDump(stdout, payload->data, payload->len);

    ptr = payload->data;
    ptr += utilUnkcapUINT16(ptr, &type);

    switch (type) {
      case ADDOSS_ID_SENSOR_DATA:
        printf("ADDOSS_ID_SENSOR_DATA unsupported\n");
        ptr += payload->len - 2;
        break;
      case ADDOSS_ID_REQ_OSG_STATUS:
        printf("ADDOSS_ID_REQ_OSG_STATUS unsupported\n");
        ptr += payload->len - 2;
        break;
      case ADDOSS_ID_OSG_STATUS:
        PrintOSG_STATUS(ptr);
        break;
      case ADDOSS_ID_REQ_OBP_STATUS:
        printf("ADDOSS_ID_REQ_OBP_STATUS unsupported\n");
        ptr += payload->len - 2;
        break;
      case ADDOSS_ID_OBP_STATUS:
        printf("ADDOSS_ID_OBP_STATUS unsupported\n");
        ptr += payload->len - 2;
        break;
      case ADDOSS_ID_SET_OBP_POWER:
        printf("ADDOSS_ID_SET_OBP_POWER unsupported\n");
        ptr += payload->len - 2;
        break;
      case ADDOSS_ID_SET_SENSOR_POWER:
        printf("ADDOSS_ID_SET_SENSOR_POWER unsupported\n");
        ptr += payload->len - 2;
        break;
      case ADDOSS_ID_REQ_SENSOR_DATA:
        printf("ADDOSS_ID_REQ_SENSOR_DATA unsupported\n");
        ptr += payload->len - 2;
        break;
      default:
        printf("Not an ADDOSS frame? Type '%d' is unrecognized.\n", type);
        ptr += payload->len - 2;
    }
}

int main(int argc, char **argv)
{
int count;
gzFile *gz;
SBD_MESSAGE message;

    if ((gz = gzdopen(fileno(stdin), "r")) == NULL) {
        perror("gzdopen");
        exit(1);
    }

    count = 0;
    while (sbdReadMessage(gz, &message)) {
        ++count;
        PrintSBD(&message);
        PrintADDOSSframe(&message.payload);
    }

    printf("%d messages read\n", count);
}

/* Revision History
 *
 * $Log: main.c,v $
 * Revision 1.1  2013/03/11 23:18:47  dechavez
 * initial release
 *
 */
