#pragma ident "$Id: service.c,v 1.1 2013/03/11 23:04:16 dechavez Exp $"
/*======================================================================
 *
 *  Service a single client connection.
 *
 *====================================================================*/
#include "sbds.h"

VOID LogRecvError(SBD *sbd)
{
int error, LogLevel;

    error = sbdGetRecvError(sbd);

    switch(error) {
      case ECONNRESET:
        LogMsg(LOG_DEBUG, "%s: connection reset by peer", sbdPeerIdent(sbd));
        LogLevel = LOG_DEBUG;
        break;

      default:
        LogLevel = LOG_INFO;
        break;
    }

    LogMsg(LogLevel, "%s: sbdRecvFrame: error=%d", sbdPeerIdent(sbd), error);
}


static VOID ProcessMessage(CLIENT *client, SBD_MESSAGE *message)
{
int len, code;
static char *fid = "ProcessMessage";

    LogMsg(LOG_INFO, "%d byte message received from %s\n", message->len, client->ident);
    SaveMessage(message);
    if (message->mt_header.valid) {
        message->mt_confirm.cdr = 0xabcdef12;
        memcpy(message->mt_confirm.myid, message->mt_header.myid, 5);
        memcpy(message->mt_confirm.imei, message->mt_header.imei, 16);
        message->mt_confirm.status = 1;
        message->mt_confirm.valid = TRUE;
        message->mt_header.valid = message->mo_header.valid = message->payload.valid = FALSE;
        sbdPackMessage(message);
        if (!sbdSendMessage(client->sbd, message)) LogMsg(LOG_INFO, "%s: error sending MT_CONFIRM: %s", fid, strerror(errno));
    }
}

/* Thread to service one client */

static THREAD_FUNC ServiceThread(void *argptr)
{
CLIENT *client;
UINT64 count = 0;
SBD_MESSAGE message;
static char *fid = "ServiceThread";

    LogMsg(LOG_DEBUG, "%s: thread started, id = 0x%x", fid, THREAD_SELF());

    client = (CLIENT *) argptr;
    client->status = SBDS_STATUS_RECV;

/* Read and process one SBD message */

    if (sbdRecvMessage(client->sbd, &message)) ProcessMessage(client, &message);

    CloseClientConnection(client);
    LogMsg(LOG_DEBUG, "%s: thread 0x%x exits", fid, THREAD_SELF());
    THREAD_EXIT((void *) 0);
}

/* Break a newly established connection w/o servicing it */

VOID BreakNewConnection(SBD *sbd)
{
    LogMsg(LOG_INFO, "%s: connection aborted", sbdPeerIdent(sbd));
    sbdClose(sbd);
}

/* Service a new connection */

VOID ServiceConnection(SBD *sbd)
{
THREAD tid;
CLIENT *client;
static CHAR *fid = "ServiceConnection";

    BlockOnShutdown();

/* Grab the next available CLIENT slot */

    if ((client = NextAvailableClient(sbd)) == NULL) {
        LogMsg(LOG_WARN, "WARNING: new connection rejected (threshold reached)");
        BreakNewConnection(sbd);
        return;
    }

/* And leave behind a thread to deal with it */

    if (!THREAD_CREATE(&tid, ServiceThread, client)) {
        LogMsg(LOG_ERR, "%s: %s: THREAD_CREATE: %s", client->ident, fid, strerror(errno));
        BreakNewConnection(sbd);
        return;
    }
    THREAD_DETACH(tid);
}

/* Revision History
 *
 * $Log: service.c,v $
 * Revision 1.1  2013/03/11 23:04:16  dechavez
 * initial release
 *
 */
