#pragma ident "$Id: write.c,v 1.1 2013/03/11 23:04:16 dechavez Exp $"
/*======================================================================
 *
 *  Write incoming messages to an ISI disk loop
 *
 *====================================================================*/
#include "sbds.h"
#include "isi.h"
#include "isi/dl.h"

#define MY_MOD_ID SBDS_MOD_DISKLOOP

static ISI_DL *dl = NULL;
static ISI_RAW_PACKET *pkt = NULL;
static ISI_DL_OPTIONS options;

VOID CloseDiskLoop(VOID)
{
    if (dl != NULL) {
        LogMsg(LOG_INFO, "closing '%s' ISI disk loop", dl->sys->site);
        isidlCloseDiskLoop(dl);
        dl = NULL;
    }
}

BOOL SaveMessage(SBD_MESSAGE *message)
{
int len;
char tstamp[256];
BOOL ok;
static char *fid = "SaveMessage";

    if (message == NULL) {
        LogMsg(LOG_INFO, "%s: *** ERROR *** NULL message received", fid);
        errno = EINVAL;
        return FALSE;
    }

    if (message->mo_header.valid) {
        utilLttostr(message->mo_header.tstamp, 0, tstamp);
        len = message->payload.valid ? message->payload.len : 0;
        LogMsg(LOG_INFO, "RECV %d MO_HEADER bytes: imei=%s cdr=0x%08x momsn=0x%04x status=%02d tstamp=%s",
            len,
            message->mo_header.imei,
            message->mo_header.cdr,
            message->mo_header.momsn,
            message->mo_header.status,
            tstamp
        );
    }

    if (message->mt_header.valid) {
        len = message->payload.valid ? message->payload.len : 0;
        LogMsg(LOG_INFO, "RECV %d MT_HEADER bytes: imei=%s myid=0x%08x flags=0x%04x",
            len,
            message->mt_header.imei,
            message->mt_header.myid,
            message->mt_header.flags
        );
    }

    if (message->mt_confirm.valid) {
        len = message->payload.valid ? message->payload.len : 0;
        LogMsg(LOG_INFO, "RECV %d MT_CONFIRM bytes: imei=%s cdr=0x%08x myid=0x%08x status=%d",
            len,
            message->mt_confirm.imei,
            message->mt_confirm.cdr,
            message->mt_confirm.myid,
            message->mt_confirm.status
        );
    }

    pkt->hdr.len.used = pkt->hdr.len.native = message->len;
    pkt->payload = message->body;
    
    BlockShutdown(fid);
        ok = isidlWriteToDiskLoop(dl, pkt, &options);
    UnblockShutdown(fid);

    if (!ok) {
        LogMsg(LOG_INFO, "%s: isidlWriteToDiskLoop: %s", fid, strerror(errno));
        return FALSE;
    }
}

VOID InitDiskLoopWriter(PARAM *par)
{
static char *fid = "InitDiskLoopWriter";


    if ((dl = isidlOpenDiskLoop(&par->glob, par->site, &par->lp, ISI_RDWR)) == NULL) {
        LogMsg(LOG_INFO, "%s: can't open `%s' disk loop", fid, par->site);
        GracefulExit(MY_MOD_ID + 1);
    }
        
    isidlInitOptions(&options);
    options.flags = ISI_OPTION_GENERATE_SEQNO;

    if ((pkt = isiAllocateRawPacket(dl->sys->maxlen)) == NULL) {
        LogMsg(LOG_INFO, "%s: isiAllocateRawPacket: %s", fid, strerror(errno));
        GracefulExit(MY_MOD_ID + 2);
    }
    strncpy(pkt->hdr.site, par->site, ISI_SITELEN);
    pkt->hdr.desc.comp  = ISI_COMP_NONE;
    pkt->hdr.desc.type  = ISI_TYPE_SBD1;
    pkt->hdr.desc.order = ISI_ORDER_UNDEF;
    pkt->hdr.desc.size  = sizeof(UINT8);
    pkt->hdr.status     = ISI_RAW_STATUS_OK;

    LogMsg(LOG_INFO, "ISI disk loop '%s' opened for writing", par->site);

    return;
}

/* Revision History
 *
 * $Log: write.c,v $
 * Revision 1.1  2013/03/11 23:04:16  dechavez
 * initial release
 *
 */
