#pragma ident "$Id: main.c,v 1.1 2013/03/11 23:01:26 dechavez Exp $"
#include "sbd.h"

static void help(char *myname)
{
    fprintf(stderr, "usage: %s { mo | mt } SBD_server\n", myname);
    exit(1);
}

static void PrintContents(SBD_MESSAGE *message)
{
    if (message->mo_header.valid) {
        printf("MO header\n");
        printf("reference number = %lu\n", message->mo_header.cdr);
        printf("            IMEI = %s\n",  message->mo_header.imei);
        printf("          status = %d\n",  message->mo_header.status);
        printf("           momsn = %hd\n", message->mo_header.momsn);
        printf("           mtmsn = %hd\n", message->mo_header.mtmsn);
        printf("          tstamp = %s\n", utilLttostr(message->mo_header.tstamp, 0, NULL));
    }

    if (message->mt_header.valid) {
        printf("MT header\n");
        printf("unique client ID = 0x%04x\n", message->mt_header.myid);
        printf("            IMEI = %s\n",     message->mt_header.imei);
        printf("           flags = 0x%04x\n", message->mt_header.flags);
    }

    if (message->mt_confirm.valid) {
        printf("MT conformation\n");
        printf("unique client ID = 0x%04x\n", message->mt_confirm.myid);
        printf("            IMEI = %s\n",     message->mt_confirm.imei);
        printf("reference number = %lu\n",    message->mt_confirm.cdr);
        printf("          status = %d\n",     message->mt_confirm.status);
    }

    if (message->payload.valid) {
        printf("%d bytes of user payload\n", message->payload.len);
        utilPrintHexDump(stdout, message->payload.data, message->payload.len);
    }
}

int main(int argc, char **argv)
{
int i, retval;
static char *server = NULL;
static int port = SBD_DEFAULT_PORT;
time_t now;
SBD *sbd;
SBD_MESSAGE message;

    if (argc != 3) help(argv[0]);
    if (strcasecmp(argv[1], "mo") == 0) {
        message.mo_header.valid  = TRUE;
        message.mt_header.valid  = FALSE;
        message.mt_confirm.valid = FALSE;
    } else if (strcasecmp(argv[1], "mt") == 0) {
        message.mo_header.valid  = FALSE;
        message.mt_header.valid  = TRUE;
        message.mt_confirm.valid = FALSE;
    } else {
        help(argv[0]);
    }

    server = argv[2];

    if ((sbd = sbdOpen(server, port, NULL, NULL, 0)) == NULL) {
        fprintf(stderr, "sbdOpen: ");
        perror(server);
        exit(1);
    }

    message.protocol = 1;

    message.mo_header.cdr    = 12345678;
    sprintf(message.mo_header.imei, "0123456789ABCDE");
    message.mo_header.status = 0;
    message.mo_header.momsn  = 1;
    message.mo_header.mtmsn  = 2;
    message.mo_header.tstamp = time(NULL);

    now = time(NULL);
    utilPackUINT32(message.mt_header.myid, now);
    sprintf(message.mt_header.imei, "300025010418970");
    message.mt_header.flags = SBD_MT_FLAG_NONE;

    sprintf(message.payload.data, "Hello, world!\n");
    message.payload.len = strlen(message.payload.data);
    message.payload.valid = TRUE;

    if ((retval = sbdPackMessage(&message)) < 0) {
        printf("sbdPackMessage: error %d\n", retval);
        exit(1);
    }

    if (sbdParseMessageBody(&message) != 0) printf("unsupported IEs ignored\n");
    printf("Outgoing message:\n");
    PrintContents(&message);

    if (!sbdSendMessage(sbd, &message)) {
        perror("sbdSendMessage");
        exit(1);
    }

    printf("%d byte message sent\n", message.len);
    utilPrintHexDump(stdout, message.body, message.len);

    if (message.mt_header.valid) {
        printf("Waiting for confirmation from %s\n", server);
        if (sbdRecvMessage(sbd, &message)) {
            printf("Reply received\n");
            if (sbdParseMessageBody(&message) != 0) printf("unsupported IEs ignored\n");
            PrintContents(&message);
        } else {
            printf("sbdRecvMessage: %s\n", strerror(errno));
        }
    }

    exit(1);
}
