#pragma ident "$Id: addoss.h,v 1.1 2013/03/07 21:04:06 dechavez Exp $"
#ifndef addoss_include_defined
#define addoss_include_defined

/* platform specific stuff */

#include "platform.h"
#include "sbd.h"

#ifdef __cplusplus
extern "C" {
#endif

/* payload idents */

#define ADDOSS_ID_SENSOR_DATA      1 
#define ADDOSS_ID_REQ_OSG_STATUS   2 
#define ADDOSS_ID_OSG_STATUS       3 
#define ADDOSS_ID_REQ_OBP_STATUS   4 
#define ADDOSS_ID_OBP_STATUS       5 
#define ADDOSS_ID_SET_OBP_POWER    6 
#define ADDOSS_ID_SET_SENSOR_POWER 7 
#define ADDOSS_ID_REQ_SENSOR_DATA  8 

/* timestamp used by OSG status messages */

typedef struct {
    UINT16 year;
    UINT8  month;
    UINT8  day;
    UINT8  hour;
    UINT8  minute;
    UINT8  second;
    UINT32 epoch; /* above values converted to Unix time */
} ADDOSS_OSG_TIMESTAMP;

/* OSG status */

#define ADDOSS_OSG_STATUS_HEARTBEAT     1
#define ADDOSS_OSG_STATUS_LOST_OBS_COMM 2
#define ADDOSS_OSG_STATUS_RECV_OBS_PING 3

typedef struct {
    UINT16               reason;              /* one of ADDOSS_OSG_STATUS_x above */
    ADDOSS_OSG_TIMESTAMP tstamp;              /* what this measures depends on reason */
    UINT16               nCAMERR;             /* number of CAMERR messages since initialization */
    UINT16               nSendPacket;         /* number of times SendPacket was called since initialization */
    UINT16               nSendPacketRetries;  /* number of SendPacket retries since initialization */
    UINT16               nPacketsSent;        /* number of successful SendPacket calls since initialization */
    UINT16               nPacketsReceived;    /* number of packets received since initialization */
    UINT8                nQueue;              /* number of messages currently in send queue */
    struct {
        UINT8  mode;                 /* mode */
        UINT8  PSKerror;             /* PSK Error Code */
        UINT8  nFramesExpected;      /* number of frames expected */
        UINT8  nCRCerrors;           /* number of frames iwth bad CRCs */
        UINT8  msesign;              /* equalizer mean squared error sign (0=pos, 1=neg) */
        UINT32 MeanSquaredError;     /* equalizer mean sqaured error */
        UINT8  rsssign;              /* received signal strength sign (0=pos, 1=neg) */
        UINT32 RecvSignalStrength;   /* received signal strength */
        UINT8  isnrsign;             /* input SNR sign (0=pos, 1=neg) */
        UINT32 InputSNR;             /* input SNR */
        UINT8  osnrsign;             /* output SNR sign (0=pos, 1=neg) */
        UINT32 OutputSNR;            /* output SNR */
        UINT8  sdnsign;              /* standard deviation of noise sign (0=pos, 1=neg) */
        UINT32 StdDevNoise;          /* standard deviation of noise */
        ADDOSS_OSG_TIMESTAMP tstamp; /* time of last packet receipt */
    } lastpacket;
} ADDOSS_OSG_STATUS;

/* function prototypes */

/* unpack.c */
int addossUnpackOSGstatus(UINT8 *start, ADDOSS_OSG_STATUS *dest);

/* version.c */
char *addossVersionString(VOID);
VERSION *addossVersion(VOID);

#ifdef __cplusplus
}
#endif

#endif /* addoss_include_defined */

/* Revision History
 *
 * $Log: addoss.h,v $
 * Revision 1.1  2013/03/07 21:04:06  dechavez
 * initial release
 *
 */
