#pragma ident "$Id: tristar.h,v 1.3 2012/05/02 18:19:33 dechavez Exp $"
/*======================================================================
 *
 *  Modbus related constants for the TriStar power controller
 *
 *====================================================================*/
#ifndef tristar_h_included
#define tristar_h_included

#include "platform.h"
#include "modbus.h"

#ifdef __cplusplus
extern "C" {
#endif

#define TRISTAR_ADDR_NONE 0xFFFF

typedef struct {
    UINT16 hi; /* if not TRISTAR_ADDR_NONE, address high order word */
    UINT16 lo; /* address of low order word */
} TRISTAR_ADDR;

typedef struct {
    TRISTAR_ADDR addr; /* address(es) with which to load register reading */
    REAL64 scaling;    /* multiplier to convert register reading to physical value (0 => bitfield) */
    INT32 raw;         /* raw register reading (INT32 in case of hi/lo addresses present) */
    REAL64 value;      /* converted register reading */
    BOOL valid;        /* TRUE when raw and value are valid */
} TRISTAR_REG;

typedef struct {
    TRISTAR_REG adc_vb_f;
    TRISTAR_REG adc_vs_f;
    TRISTAR_REG adc_vx_f;
    TRISTAR_REG adc_ipv_f;
    TRISTAR_REG adc_iload_f;
    TRISTAR_REG Vb_f;
    TRISTAR_REG T_hs;
    TRISTAR_REG T_batt;
    TRISTAR_REG V_ref;
    TRISTAR_REG Ah_r;
    TRISTAR_REG Ah_t;
    TRISTAR_REG Hourmeter;
    TRISTAR_REG Alarm;
    TRISTAR_REG fault;
    TRISTAR_REG dip_switch;
    TRISTAR_REG control_mode;
    TRISTAR_REG control_state;
    TRISTAR_REG load_state;
    TRISTAR_REG d_filt;
    TRISTAR_REG kwh;
    TRISTAR_REG min_batt_v;
    TRISTAR_REG max_batt_v;
    struct {
        char string[9];
        BOOL valid;
    } serialno;
} TRISTAR_RAM;

#define TRISTAR_ADDR_ADC_VB_F     0x0008
#define TRISTAR_ADDR_ADC_VS_F     0x0009
#define TRISTAR_ADDR_ADC_VX_F     0x000A
#define TRISTAR_ADDR_ADC_IPV_F    0x000B
#define TRISTAR_ADDR_ILOAD_F      0x000C
#define TRISTAR_ADDR_VB_F         0x000D
#define TRISTAR_ADDR_T_HS         0x000E
#define TRISTAR_ADDR_T_BATT       0x000F
#define TRISTAR_ADDR_V_REF        0x0010
#define TRISTAR_ADDR_AH_R_HI      0x0011
#define TRISTAR_ADDR_AH_R_LO      0x0012
#define TRISTAR_ADDR_AT_T_HI      0x0013
#define TRISTAR_ADDR_AT_T_LO      0x0014
#define TRISTAR_ADDR_HOURMETER_HI 0x0015
#define TRISTAR_ADDR_HOURMETER_LO 0x0016
#define TRISTAR_ADDR_ALARM_HI     0x001D
#define TRISTAR_ADDR_ALARM_LO     0x0017
#define TRISTAR_ADDR_FAULT        0x0018
#define TRISTAR_ADDR_DIP_SWITCH   0x0019
#define TRISTAR_ADDR_MODE         0x001A
#define TRISTAR_ADDR_STATE        0x001B
#define TRISTAR_ADDR_D_FILT       0x001C
#define TRISTAR_ADDR_KWH          0x001E
#define TRISTAR_ADDR_MIN_BATT_V   0x001F
#define TRISTAR_ADDR_MAX_BATT_V   0x0020

#define TRISTAR_REG_adc_vb_f      {{ TRISTAR_ADDR_NONE, 0x0008 }, 0.002950042724609375, 0, 0.0 }
#define TRISTAR_REG_adc_vs_f      {{ TRISTAR_ADDR_NONE, 0x0009 }, 0.002950042724609375, 0, 0.0 }
#define TRISTAR_REG_adc_vx_f      {{ TRISTAR_ADDR_NONE, 0x000a }, 0.00424652099609375,  0, 0.0 }
#define TRISTAR_REG_adc_ipv_f     {{ TRISTAR_ADDR_NONE, 0x000b }, 0.002034515380859375, 0, 0.0 }
#define TRISTAR_REG_adc_iload_f   {{ TRISTAR_ADDR_NONE, 0x000c }, 0.00966400146484375,  0, 0.0 }
#define TRISTAR_REG_Vb_f          {{ TRISTAR_ADDR_NONE, 0x000d }, 0.002950042724609375, 0, 0.0 }
#define TRISTAR_REG_T_hs          {{ TRISTAR_ADDR_NONE, 0x000e }, 1.0,                  0, 0.0 }
#define TRISTAR_REG_T_batt        {{ TRISTAR_ADDR_NONE, 0x000f }, 1.0,                  0, 0.0 }
#define TRISTAR_REG_V_ref         {{ TRISTAR_ADDR_NONE, 0x0010 }, 0.002950042724609375, 0, 0.0 }
#define TRISTAR_REG_Ah_r          {{            0x0011, 0x0012 }, 0.1,                  0, 0.0 }
#define TRISTAR_REG_Ah_t          {{            0x0013, 0x0014 }, 0.1,                  0, 0.0 }
#define TRISTAR_REG_Hourmeter     {{            0x0015, 0x0016 }, 1.0,                  0, 0.0 }
#define TRISTAR_REG_Alarm         {{            0x001D, 0x0017 }, 0.0,                  0, 0.0 }
#define TRISTAR_REG_fault         {{ TRISTAR_ADDR_NONE, 0x0018 }, 0.0,                  0, 0.0 }
#define TRISTAR_REG_dip_switch    {{ TRISTAR_ADDR_NONE, 0x0019 }, 0.0,                  0, 0.0 }
#define TRISTAR_REG_control_mode  {{ TRISTAR_ADDR_NONE, 0x001A }, 1.0,                  0, 0.0 }
#define TRISTAR_REG_control_state {{ TRISTAR_ADDR_NONE, 0x001B }, 1.0,                  0, 0.0 }
#define TRISTAR_REG_load_state    {{ TRISTAR_ADDR_NONE, 0x001B }, 1.0,                  0, 0.0 }
#define TRISTAR_REG_d_filt        {{ TRISTAR_ADDR_NONE, 0x001C }, 0.392156862745098,    0, 0.0 }
#define TRISTAR_REG_kwh           {{ TRISTAR_ADDR_NONE, 0x001E }, 1.0,                  0, 0.0 }
#define TRISTAR_REG_min_batt_v    {{ TRISTAR_ADDR_NONE, 0x001F }, 0.002950042724609375, 0, 0.0 }
#define TRISTAR_REG_max_batt_v    {{ TRISTAR_ADDR_NONE, 0x0020 }, 0.002950042724609375, 0, 0.0 }

static TRISTAR_RAM TRISTAR_RAM_DEFAULT = {
    TRISTAR_REG_adc_vb_f,
    TRISTAR_REG_adc_vs_f,
    TRISTAR_REG_adc_vx_f,
    TRISTAR_REG_adc_ipv_f,
    TRISTAR_REG_adc_iload_f,
    TRISTAR_REG_Vb_f,
    TRISTAR_REG_T_hs,
    TRISTAR_REG_T_batt,
    TRISTAR_REG_V_ref,
    TRISTAR_REG_Ah_r,
    TRISTAR_REG_Ah_t,
    TRISTAR_REG_Hourmeter,
    TRISTAR_REG_Alarm,
    TRISTAR_REG_fault,
    TRISTAR_REG_dip_switch,
    TRISTAR_REG_control_mode,
    TRISTAR_REG_control_state,
    TRISTAR_REG_load_state,
    TRISTAR_REG_d_filt,
    TRISTAR_REG_kwh,
    TRISTAR_REG_min_batt_v,
    TRISTAR_REG_max_batt_v,
    {{ '?', '?', '?', '?', '?', '?', '?', '?', 0}, FALSE}
};

#define TRISTAR_ADDR_SERIAL01 0xF000
#define TRISTAR_ADDR_SERIAL23 0xF001
#define TRISTAR_ADDR_SERIAL45 0xF002
#define TRISTAR_ADDR_SERIAL67 0xF003

#define TRISTAR_COIL_EQTRIG         0x0000
#define TRISTAR_COIL_DISCONNECT     0x0001
#define TRISTAR_COIL_CLEAR_AH_R     0x0010
#define TRISTAR_COIL_CLEAR_AH_T     0x0011
#define TRISTAR_COIL_CLEAR_KWH      0x0012
#define TRISTAR_COIL_RESET_BATT_SVC 0x0013
#define TRISTAR_COIL_CLEAR_FAULTS   0x0014
#define TRISTAR_COIL_CLEAR_ALARMS   0x0015
#define TRISTAR_COIL_LVD_OVERRIDE   0x0017
#define TRISTAR_COIL_RESET          0x00FF

/* function prototypes */

/* print.c */
void tristarPrintRam(int slave, TRISTAR_RAM *ram);

/* read.c */
BOOL tristarReadRegister(modbus_t *ctx, TRISTAR_REG *reg);
BOOL tristarReadSerial(modbus_t *ctx, char *dest);
void tristarReadRam(modbus_t *ctx, int slave, TRISTAR_RAM *dest);
void tristarReadRamLogset(modbus_t *ctx, int slave, TRISTAR_RAM *dest);

/* version.c */

VERSION *tristarVersion(void);
char *tristarVersionString(void);

#ifdef __cplusplus
}
#endif

#endif /* tristar_h_included */

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2011 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: tristar.h,v $
 * Revision 1.3  2012/05/02 18:19:33  dechavez
 * added valid field to TRISTAR_REG and serialno
 *
 * Revision 1.2  2012/04/25 21:01:37  dechavez
 * added function prototypes
 *
 * Revision 1.1  2011/10/14 17:50:53  dechavez
 * initial version
 *
 */
