#pragma ident "$Id: unpack.c,v 1.1 2013/03/07 21:03:41 dechavez Exp $"
/*======================================================================
 * 
 * Unpack ADDOSS frames from little-endian order into host order
 *
 *====================================================================*/
#include "addoss.h"
#include "util.h"

static int UnkcapOSGtstamp(UINT8 *start, ADDOSS_OSG_TIMESTAMP *dest)
{
UINT32 julianday;
UINT8 *ptr;
struct css_date_time dt;

    ptr = start;
    ptr += utilUnkcapUINT16(ptr, &dest->year);
    dest->month  = *ptr++;
    dest->day    = *ptr++;
    dest->hour   = *ptr++;
    dest->minute = *ptr++;
    dest->second = *ptr++;

    julianday = utilYmdtojd(dest->year, dest->month, dest->day);
    dt.date = dest->year * 1000 + julianday;
    dt.hour = dest->hour;
    dt.minute = dest->minute;
    dt.second = (float) dest->second;
    util_htoe(&dt);
    dest->epoch = (UINT32) dt.epoch;

    return (int) (ptr - start);
}

static void PrintOffset(UINT8 *start, UINT8 *ptr, char *where)
{
    printf("%s offset = %d\n", where, (int) (ptr - start) + 2);
}

int addossUnpackOSGstatus(UINT8 *start, ADDOSS_OSG_STATUS *dest)
{
UINT8 *ptr;

    ptr = start;
    ptr += utilUnkcapUINT16(ptr, &dest->reason);
    ptr += UnkcapOSGtstamp(ptr, &dest->tstamp);
    ptr += utilUnkcapUINT16(ptr, &dest->nCAMERR);
    ptr += utilUnkcapUINT16(ptr, &dest->nSendPacket);
    ptr += utilUnkcapUINT16(ptr, &dest->nSendPacketRetries);
    ptr += utilUnkcapUINT16(ptr, &dest->nPacketsSent);
    ptr += utilUnkcapUINT16(ptr, &dest->nPacketsReceived);
    dest->nQueue = *ptr++;
    dest->lastpacket.mode = *ptr++;
    dest->lastpacket.PSKerror = *ptr++;
    dest->lastpacket.nFramesExpected = *ptr++;
    dest->lastpacket.nCRCerrors = *ptr++;
    dest->lastpacket.msesign = *ptr++;
    ptr += utilUnkcapUINT32(ptr, &dest->lastpacket.MeanSquaredError);
    dest->lastpacket.rsssign = *ptr++;
    ptr += utilUnkcapUINT32(ptr, &dest->lastpacket.RecvSignalStrength);
    dest->lastpacket.isnrsign = *ptr++;
    ptr += utilUnkcapUINT32(ptr, &dest->lastpacket.InputSNR);
    dest->lastpacket.osnrsign = *ptr++;
    ptr += utilUnkcapUINT32(ptr, &dest->lastpacket.OutputSNR);
    dest->lastpacket.sdnsign = *ptr++;
    ptr += utilUnkcapUINT32(ptr, &dest->lastpacket.StdDevNoise);
    ptr += UnkcapOSGtstamp(ptr, &dest->lastpacket.tstamp);

    return (int) (ptr - start);
}

/* Revision History
 *
 * $Log: unpack.c,v $
 * Revision 1.1  2013/03/07 21:03:41  dechavez
 * initial release
 *
 */
