#pragma ident "$Id: bud.c,v 1.1 2011/11/07 17:30:52 dechavez Exp $"
/*======================================================================
 *
 * Tee incoming ISI data with MiniSEED headers into a BUD tree
 *
 *====================================================================*/
#include "isi/dl.h"

void isidlWriteToBud(ISI_DL_BUD *bud, LOGIO *lp, ISI_RAW_PACKET *raw)
{
FILE *fp;
UINT8 *ptr;
LISS_PKT pkt;
LISS_FSDH *fsdh;
int year, day, unused, len;
char path[MAXPATHLEN+1];
static char *fid = "isidlWriteToBud";

    if (bud == NULL || raw == NULL) return;

/* BUD archiving is enabled/disabled via existance of the root dir */

    if (!utilDirectoryExists(bud->root)) return;

/* Nothing to do if not MiniSEED or we don't have an equivalent MiniSEED header */

    if (raw->hdr.desc.type == ISI_TYPE_MSEED) {
        lissUnpackMiniSeed(&pkt, raw->payload, 0);
    } else if (raw->hdr.mseed[0] != 0) {
        lissUnpackMiniSeed(&pkt, raw->hdr.mseed, 0);
    } else {
        return;
    }

/* We are only interested in FSDH here */

    fsdh = &pkt.hdr.fsdh;
    utilTsplit(fsdh->start, &year, &day, &unused, &unused, &unused, &unused);

/* Massage the location code to ensure no blanks in the file name */

    if (strcmp(fsdh->locid, "  ") == 0) fsdh->locid[0] = 0;
    if (fsdh->locid[1] == ' ') fsdh->locid[1] = 0;
    if (fsdh->locid[0] == ' ') {
        fsdh->locid[0] = fsdh->locid[1];
        fsdh->locid[1] = 0;
    }

/* Ensure that all the required subdirectories exist */

    sprintf(path, "%s/%s/%s", bud->root, fsdh->netid, fsdh->staid);
    if (util_mkpath(path, 0775) != 0) {
        if (!bud->error) logioMsg(lp, LOG_ERR, "*** BUD ERROR *** %s: mkdir %s: %s", fid, path, strerror(errno));
        bud->error = TRUE;
        return;
    }

/* Open the appropriate file */

    sprintf(path+strlen(path), "/%s.%s.%s.%s.%04d.%04d", fsdh->staid, fsdh->netid, fsdh->locid, fsdh->chnid, year, day);
    if ((fp = fopen(path, "a")) == NULL) {
        if (!bud->error) logioMsg(lp, LOG_ERR, "*** BUD ERROR *** %s: open %s: %s", fid, path, strerror(errno));
        bud->error = TRUE;
        return;
    }

/* Non-native MiniSEED get converted by replacing the header */

    if (raw->hdr.desc.type != ISI_TYPE_MSEED) {
        if (fwrite(raw->hdr.mseed, sizeof(UINT8), ISI_MSEED_HDR_LEN, fp) != ISI_MSEED_HDR_LEN) {
            if (!bud->error) logioMsg(lp, LOG_ERR, "*** BUD ERROR *** %s: write (hdr) %s: %s", fid, path, strerror(errno));
            bud->error = TRUE;
        }
        ptr = raw->payload + ISI_MSEED_HDR_LEN;
        len = raw->hdr.len.used - ISI_MSEED_HDR_LEN;
    } else {
        ptr = raw->payload;
        len = raw->hdr.len.used;
    }

    if (fwrite(ptr, sizeof(UINT8), len, fp) != len) {
        if (!bud->error) logioMsg(lp, LOG_ERR, "*** BUD ERROR *** %s: write (payload) %s: %s", fid, path, strerror(errno));
        bud->error = TRUE;
    }

    bud->error = FALSE;
    fclose(fp);
    return;
}

/* Revision History
 *
 * $Log: bud.c,v $
 * Revision 1.1  2011/11/07 17:30:52  dechavez
 * created: isidlWriteToBud()
 *
 */
