#pragma ident "$Id: register.c,v 1.7 2011/02/01 19:45:08 dechavez Exp $"
/*======================================================================
 * 
 * Connect to a Q330
 *
 *====================================================================*/
#include "q330.h"

QDP *q330Register(Q330_ADDR *addr, int port, int debug, LOGIO *lp, int *errcode)
{
QDP *qdp;
QDP_PAR par;
static char *fid = "q330Register";

    if (addr == NULL) {
        errno = EINVAL;
        return NULL;
    }

    if (!qdpValidPort(port)) {
        errno = EINVAL;
        return NULL;
    }

    qdpInitPar(&par, port);
    qdpSetConnect(&par, &addr->connect);
    qdpSetSerialno(&par, addr->serialno);
    qdpSetAuthcode(&par, addr->authcode);
    qdpSetDebug(&par, debug);

    if ((qdp = qdpConnect(&par, lp, errcode)) == NULL) return NULL;
    addr->serialno = qdp->par.serialno;

    return qdp;
}

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2007 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: register.c,v $
 * Revision 1.7  2011/02/01 19:45:08  dechavez
 * update calling addr with serial number after registration
 *
 * Revision 1.6  2011/01/25 18:07:26  dechavez
 * qdpSetConnect() replaces qdpSetIdent()
 *
 * Revision 1.5  2011/01/14 00:28:53  dechavez
 * qdpSetHost() to qdpSetIdent(), qdpConnectWithPar() to qdpConnect()
 *
 * Revision 1.4  2010/12/23 21:44:31  dechavez
 * use qdpValidPort() to test for input validity
 *
 * Revision 1.3  2010/03/31 20:23:42  dechavez
 * don't set client side port numbers (qdpSetMyXPort functions gone)
 *
 * Revision 1.2  2010/03/22 21:43:07  dechavez
 * added errcode arg required by libqdp 1.11.0
 *
 * Revision 1.1  2007/10/31 17:16:31  dechavez
 * initial release
 *
 */
