#pragma ident "$Id: udp.c,v 1.2 2011/01/31 18:16:16 dechavez Exp $"
/*========================================================================
 *
 * UDP header stuff
 *
 *======================================================================*/
#include "qio.h"

int qioPackUdpHeader(UINT8 *start, QIO *qio, UINT8 *iphdr, int port, UINT8 *payload, int len)
{
UINT8 *ptr;
UINT32 srcip, dstip;

    utilUnpackUINT32(&iphdr[12], &srcip);
    utilUnpackUINT32(&iphdr[16], &dstip);

    ptr = start;
    ptr += utilPackUINT16(ptr, qio->xmit.port);
    ptr += utilPackUINT16(ptr, port);
    ptr += utilPackUINT16(ptr, len + QIO_UDP_HDRLEN);
    ptr += utilPackUINT16(ptr, htons(utilUdpChecksum(start, srcip, dstip, payload, len)));

    return (int) (start - ptr);
}

BOOL qioVerifyUdpHeader(UINT8 *start, UINT8 *iphdr, UINT8 *payload, int len)
{
#define UDP_CHECKSUM_OFFSET 6
UINT16 expected, actual;
UINT32 srcip, dstip;

    utilUnpackUINT32(&iphdr[12], &srcip);
    utilUnpackUINT32(&iphdr[16], &dstip);

    utilUnpackUINT16(&start[UDP_CHECKSUM_OFFSET], &expected);
    actual = htons(utilUdpChecksum(start, srcip, dstip, payload, len));

    return (actual == expected) ? TRUE : FALSE;
}

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2011 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: udp.c,v $
 * Revision 1.2  2011/01/31 18:16:16  dechavez
 * removed checksum fail message
 *
 * Revision 1.1  2011/01/25 18:35:12  dechavez
 * initial release, checksum mismatches debugged to stdout
 *
 * Revision 1.1  2011/01/12 23:19:34  dechavez
 * created
 *
 */
