#pragma ident "$Id: pack.c,v 1.2 2013/03/11 23:00:11 dechavez Exp $"
/*======================================================================
 * 
 * Pack SBD Information Elements
 *
 *====================================================================*/
#include "sbd.h"

int sbdPackMO_HEADER(UINT8 *start, SBD_MO_HEADER *src)
{
UINT8 *ptr;
UINT16 len = 28;

    if (start == NULL || src == NULL || !src->valid) return 0;

    ptr = start;
    *ptr++ = SBD_IEI_MO_HEADER;
    ptr += utilPackUINT16(ptr, len);
    ptr += utilPackUINT32(ptr, src->cdr);
    ptr += utilPackBytes(ptr, src->imei, 15);
    *ptr++ = src->status;
    ptr += utilPackUINT16(ptr, src->momsn);
    ptr += utilPackUINT16(ptr, src->mtmsn);
    ptr += utilPackUINT32(ptr, src->tstamp);

    return (int) (ptr - start);

}

int sbdPackMT_HEADER(UINT8 *start, SBD_MT_HEADER *src)
{
UINT8 *ptr;
UINT16 len = 21;

    if (start == NULL || src == NULL || !src->valid) return 0;

    ptr = start;
    *ptr++ = SBD_IEI_MT_HEADER;
    ptr += utilPackUINT16(ptr, len);
    ptr += utilPackBytes(ptr, src->myid, 4);
    ptr += utilPackBytes(ptr, src->imei, 15);
    ptr += utilPackUINT16(ptr, src->flags);

    return (int) (ptr - start);

}

int sbdPackMT_CONFIRM(UINT8 *start, SBD_MT_CONFIRM *src)
{
UINT8 *ptr;
UINT16 len = 25;

    if (start == NULL || src == NULL || !src->valid) return 0;

    ptr = start;
    *ptr++ = SBD_IEI_MT_CONFIRM;
    ptr += utilPackUINT16(ptr, len);
    ptr += utilPackBytes(ptr, src->myid, 4);
    ptr += utilPackBytes(ptr, src->imei, 15);
    ptr += utilPackUINT32(ptr, src->cdr);
    ptr += utilPackUINT16(ptr, src->status);

    return (int) (ptr - start);

}

int sbdPackPayload(UINT8 *start, SBD_PAYLOAD *src, int which)
{
UINT8 *ptr;
UINT16 len = 21;

    if (start == NULL || src == NULL || !src->valid) return 0;

    ptr = start;
    *ptr++ = which;
    len = (src->len <= SBD_MAX_MSGLEN) ? src->len : SBD_MAX_MSGLEN;
    ptr += utilPackUINT16(ptr, len);
    ptr += utilPackBytes(ptr, src->data, len);
    
    return (int) (ptr - start);
}

int sbdPackMessage(SBD_MESSAGE *message)
{
UINT8 *ptr, *lenptr;

    if (message == NULL) {
        errno = EINVAL;
        return -1;
    }

/* Pack the approriate header, and payload if applicable */

    ptr = message->body;
    *ptr++ = 1; /* always protocol 1 */
    lenptr = ptr; ptr += 2; /* skip over length field */
    if (message->mo_header.valid) {
        ptr += sbdPackMO_HEADER(ptr, &message->mo_header);
        if (message->payload.valid)   ptr += sbdPackPayload(ptr, &message->payload, SBD_IEI_MO_PAYLOAD);
    }
    if (message->mt_header.valid) {
        ptr += sbdPackMT_HEADER(ptr, &message->mt_header);
        if (message->payload.valid)   ptr += sbdPackPayload(ptr, &message->payload, SBD_IEI_MT_PAYLOAD);
    }
    if (message->mt_confirm.valid) ptr += sbdPackMT_CONFIRM(ptr, &message->mt_confirm);

    message->len = (UINT16) (ptr - message->body); /* full body, including preamble */
    utilPackUINT16(lenptr, message->len - SBD_PROTOCOL_1_PREAMBLE_LEN); /* length that is sent does not include preamble */

    return (int) message->len; /* total length, including preamble */
}

/* Revision History
 *
 * $Log: pack.c,v $
 * Revision 1.2  2013/03/11 23:00:11  dechavez
 * added SBD_MT_CONFIRM suppor
 *
 * Revision 1.1  2013/03/07 21:01:11  dechavez
 * initial release
 *
 */
