#pragma ident "$Id: parse.c,v 1.2 2013/03/11 22:58:55 dechavez Exp $"
/*======================================================================
 * 
 * Parse a raw SBD message
 *
 *====================================================================*/
#include "sbd.h"
#include "util.h"

static void ResetMessageIEs(SBD_MESSAGE *message)
{
    message->mo_header.valid = FALSE;
    message->mt_header.valid = FALSE;
    message->mt_confirm.valid = FALSE;
    message->payload.valid = FALSE;
}

int sbdParseMessageBody(SBD_MESSAGE *message)
{
UINT16 len;
UINT8 *ptr, *eom, iei;
static char *fid = "sbdParseMessageBody";

    if (message == NULL) return SBD_ERROR;
    ResetMessageIEs(message);

    ptr = message->body;
    if ((message->protocol = *ptr++) != 1) return SBD_EPROTO;
    utilUnpackUINT16(ptr, &message->len);
    if (message->len > SBD_MAX_MSGLEN) return SBD_TOOBIG;

    ptr = message->body + SBD_PROTOCOL_1_PREAMBLE_LEN;
    eom = message->body + message->len;

    while (ptr < eom) {
        iei = *ptr++;
        switch (iei) {
          case SBD_IEI_MO_HEADER:
            ptr += sbdUnpackMO_HEADER(ptr, &message->mo_header);
            break;
          case SBD_IEI_MT_HEADER:
            ptr += sbdUnpackMT_HEADER(ptr, &message->mt_header);
            break;
          case SBD_IEI_MT_CONFIRM:
            ptr += sbdUnpackMT_CONFIRM(ptr, &message->mt_confirm);
            break;
          case SBD_IEI_MO_PAYLOAD:
          case SBD_IEI_MT_PAYLOAD:
            ptr += sbdUnpackPayload(ptr, &message->payload);
            break;
          case SBD_IEI_MO_LOCATION:
          case SBD_IEI_MO_CONFIRM:
          case SBD_IEI_MT_PRIORITY:
          default:
            ptr += utilUnpackUINT16(ptr, &len);
            ptr += len; /* skip over unspported IEs */
        }
    }

    message->len += SBD_PROTOCOL_1_PREAMBLE_LEN; /* since we include the preamble in the body */

    return SBD_OK;
}

/* Revision History
 *
 * $Log: parse.c,v $
 * Revision 1.2  2013/03/11 22:58:55  dechavez
 * added SBD_MT_CONFIRM support
 *
 * Revision 1.1  2013/03/07 21:01:11  dechavez
 * initial release
 *
 */

