#pragma ident "$Id: unpack.c,v 1.2 2013/03/11 22:57:43 dechavez Exp $"
/*======================================================================
 * 
 * Unpack SBD Information Elements
 *
 *====================================================================*/
#include "sbd.h"

int sbdUnpackMO_HEADER(UINT8 *start, SBD_MO_HEADER *dest)
{
UINT8 *ptr;
UINT16 len;

    dest->valid = FALSE; /* assume failure */

    ptr = start;
    ptr += utilUnpackUINT16(ptr, &len);
    if (len != 28) return (int) (ptr + 28);
    ptr += utilUnpackUINT32(ptr, &dest->cdr);
    ptr += utilUnpackBytes(ptr, dest->imei, 15); dest->imei[15] = 0;
    dest->status = *ptr++;
    ptr += utilUnpackUINT16(ptr, &dest->momsn);
    ptr += utilUnpackUINT16(ptr, &dest->mtmsn);
    ptr += utilUnpackUINT32(ptr, &dest->tstamp);
    
    dest->valid = TRUE;
    return (int) (ptr - start);
}

int sbdUnpackMT_HEADER(UINT8 *start, SBD_MT_HEADER *dest)
{
UINT8 *ptr;
UINT16 len;

    dest->valid = FALSE; /* assume failure */

    ptr = start;
    ptr += utilUnpackUINT16(ptr, &len);
    if (len != 21) return (int) (ptr + 21);
    ptr += utilUnpackBytes(ptr, dest->myid,  4); dest->myid[ 4] = 0;
    ptr += utilUnpackBytes(ptr, dest->imei, 15); dest->imei[15] = 0;
    ptr += utilUnpackUINT16(ptr, &dest->flags);
    
    dest->valid = TRUE;
    return (int) (ptr - start);
}

int sbdUnpackMT_CONFIRM(UINT8 *start, SBD_MT_CONFIRM *dest)
{
UINT8 *ptr;
UINT16 len;

    dest->valid = FALSE; /* assume failure */

    ptr = start;
    ptr += utilUnpackUINT16(ptr, &len);
    if (len != 25) return (int) (ptr + 25);
    ptr += utilUnpackBytes(ptr, dest->myid,  4); dest->myid[ 4] = 0;
    ptr += utilUnpackBytes(ptr, dest->imei, 15); dest->imei[15] = 0;
    ptr += utilUnpackUINT32(ptr, &dest->cdr);
    ptr += utilUnpackUINT16(ptr, &dest->status);
    
    dest->valid = TRUE;
    return (int) (ptr - start);
}

int sbdUnpackPayload(UINT8 *start, SBD_PAYLOAD *dest)
{
UINT8 *ptr;
UINT16 len;

    dest->valid = FALSE; /* assume failure */

    ptr = start;
    ptr += utilUnpackUINT16(ptr, &dest->len);
    if (dest->len > SBD_MAX_MSGLEN) return (int) (ptr + len);
    ptr += utilUnpackBytes(ptr, dest->data, dest->len);
    
    dest->valid = TRUE;
    return (int) (ptr - start);
}

/* Revision History
 *
 * $Log: unpack.c,v $
 * Revision 1.2  2013/03/11 22:57:43  dechavez
 * added sbdUnpackMT_CONFIRM()
 *
 * Revision 1.1  2013/03/07 21:01:11  dechavez
 * initial release
 *
 */
