#pragma ident "$Id: print.c,v 1.2 2012/05/02 18:25:05 dechavez Exp $"
/*======================================================================
 *
 *  Print readings
 *
 *====================================================================*/
#include "tristar.h"

static void PrintAlarms(UINT32 bitfield)
{
    if (bitfield == 0) printf(" no alarms");
    if (bitfield & 0x00000001) printf(" RTSopen");
    if (bitfield & 0x00000002) printf(" RTSshort");
    if (bitfield & 0x00000004) printf(" RTSdiscon");
    if (bitfield & 0x00000008) printf(" HeatsinkSensorOpen");
    if (bitfield & 0x00000010) printf(" HeatsinkSensorShort");
    if (bitfield & 0x00000020) printf(" TristarHOT");
    if (bitfield & 0x00000040) printf(" CurrentLimit");
    if (bitfield & 0x00000080) printf(" CurrentOffset");
    if (bitfield & 0x00000100) printf(" BatterySense");
    if (bitfield & 0x00000200) printf(" BatterySenseDiscon");
    if (bitfield & 0x00000400) printf(" Uncalibrated");
    if (bitfield & 0x00000800) printf(" RTSmiswire");
    if (bitfield & 0x00001000) printf(" HiVdiscon");
    if (bitfield & 0x00002000) printf(" DiverNearMax");
    if (bitfield & 0x00004000) printf(" SystemMiswire");
    if (bitfield & 0x00008000) printf(" MOSFETOpen");
    if (bitfield & 0x00010000) printf(" P12VrefOff");
    if (bitfield & 0x00020000) printf(" LoadDiscon");
    if (bitfield & 0x00040000) printf(" Alarm19");
    if (bitfield & 0x00080000) printf(" Alarm20");
    if (bitfield & 0x00100000) printf(" Alarm21");
    if (bitfield & 0x00200000) printf(" Alarm22");
    if (bitfield & 0x00400000) printf(" Alarm23");
    if (bitfield & 0x00800000) printf(" Alarm24");
    if (bitfield & 0x01000000) printf(" Alarm25");
    if (bitfield & 0x02000000) printf(" Alarm26");
    if (bitfield & 0x04000000) printf(" Alarm27");
    if (bitfield & 0x08000000) printf(" Alarm28");
    if (bitfield & 0x10000000) printf(" Alarm29");
    if (bitfield & 0x20000000) printf(" Alarm30");
    if (bitfield & 0x40000000) printf(" Alarm31");
    if (bitfield & 0x80000000) printf(" Alarm32");
}

static void PrintFaults(UINT32 bitfield)
{
    if (bitfield == 0) printf(" no faults");
    if (bitfield & 0x00000001) printf(" Fault01");
    if (bitfield & 0x00000002) printf(" Fault02");
    if (bitfield & 0x00000004) printf(" Fault03");
    if (bitfield & 0x00000008) printf(" Fault04");
    if (bitfield & 0x00000010) printf(" Fault05");
    if (bitfield & 0x00000020) printf(" Fault06");
    if (bitfield & 0x00000040) printf(" Fault07");
    if (bitfield & 0x00000080) printf(" Fault08");
    if (bitfield & 0x00000100) printf(" Fault09");
    if (bitfield & 0x00000200) printf(" Fault10");
    if (bitfield & 0x00000400) printf(" Fault11");
    if (bitfield & 0x00000800) printf(" Fault12");
    if (bitfield & 0x00001000) printf(" Fault13");
    if (bitfield & 0x00002000) printf(" Fault14");
    if (bitfield & 0x00004000) printf(" Fault15");
    if (bitfield & 0x00008000) printf(" Fault16");
    if (bitfield & 0x00010000) printf(" Fault17");
    if (bitfield & 0x00020000) printf(" Fault18");
    if (bitfield & 0x00040000) printf(" Fault19");
    if (bitfield & 0x00080000) printf(" Fault20");
    if (bitfield & 0x00100000) printf(" Fault21");
    if (bitfield & 0x00200000) printf(" Fault22");
    if (bitfield & 0x00400000) printf(" Fault23");
    if (bitfield & 0x00800000) printf(" Fault24");
    if (bitfield & 0x01000000) printf(" Fault25");
    if (bitfield & 0x02000000) printf(" Fault26");
    if (bitfield & 0x04000000) printf(" Fault27");
    if (bitfield & 0x08000000) printf(" Fault28");
    if (bitfield & 0x10000000) printf(" Fault29");
    if (bitfield & 0x20000000) printf(" Fault30");
    if (bitfield & 0x40000000) printf(" Fault31");
    if (bitfield & 0x80000000) printf(" Fault32");
}

static void PrintSwitch(UINT8 bitfield)
{
    printf("1=%s", bitfield & 0x01 ? "ON" : "off"); printf(" ");
    printf("2=%s", bitfield & 0x02 ? "ON" : "off"); printf(" ");
    printf("3=%s", bitfield & 0x04 ? "ON" : "off"); printf(" ");
    printf("4=%s", bitfield & 0x08 ? "ON" : "off"); printf(" ");
    printf("5=%s", bitfield & 0x10 ? "ON" : "off"); printf(" ");
    printf("6=%s", bitfield & 0x20 ? "ON" : "off"); printf(" ");
    printf("7=%s", bitfield & 0x40 ? "ON" : "off"); printf(" ");
    printf("8=%s", bitfield & 0x80 ? "ON" : "off");
}

static void PrintChrg(TRISTAR_RAM *ram)
{
    printf("           Control State = ");
    if (ram->control_state.valid) {
        switch (ram->control_state.raw) {
          case 0: printf("Start\n");       break;
          case 1: printf("Night Check\n"); break;
          case 2: printf("Disconnect\n");  break;
          case 3: printf("Night\n");       break;
          case 4: printf("Fault\n");       break;
          case 5: printf("Bulk\n");        break;
          case 6: printf("PWM\n");         break;
          case 7: printf("Float\n");       break;
          case 8: printf("Equalize\n");    break;
          default:
            printf("%d?\n", ram->control_state.raw); break;
        }
    } else {
        printf("n/a\n");
    }

    printf("  Battery Voltage (Slow) = ");
    if (ram->adc_vb_f.valid) {
        printf("%5.2lf V\n", ram->adc_vb_f.value);
    } else {
        printf("n/a\n");
    }

    printf("   Battery Sense Voltage = ");
    if (ram->adc_vs_f.valid) {
        printf("%5.2lf V\n", ram->adc_vs_f.value);
    } else {
        printf("n/a\n");
    }

    printf("     Battery Temperature = ");
    if (ram->T_batt.valid) {
        printf("%d C\n", ram->T_batt.raw);
    } else {
        printf("n/a\n");
    }

    printf("      Array/Load Voltage = ");
    if (ram->adc_vx_f.valid) {
        printf("%5.2lf V\n", ram->adc_vx_f.value);
    } else {
        printf("n/a\n");
    }

    printf("          Charge Current = ");
    if (ram->adc_ipv_f.valid) {
        printf("%5.2lf A\n", ram->adc_ipv_f.value);
    } else {
        printf("n/a\n");
    }
}

static void PrintLoad(TRISTAR_RAM *ram)
{
    printf("              Load State = ");
    if (ram->load_state.valid) {
        switch (ram->load_state.raw) {
          case 0: printf("Start\n");        break;
          case 1: printf("Normal\n");       break;
          case 2: printf("LVD Warning\n");  break;
          case 3: printf("LVD\n");          break;
          case 4: printf("Fault\n");        break;
          case 5: printf("Disconnect\n");   break;
          case 6: printf("Normal off\n");   break;
          case 7: printf("Override LVD\n"); break;
          default:
            printf("%d?\n", ram->load_state.raw); break;
        }
    } else {
        printf("n/a\n");
    }

    printf("      Array/Load Voltage = ");
    if (ram->adc_vx_f.valid) {
        printf("%5.2lf V\n", ram->adc_vx_f.value);
    } else {
        printf("n/a\n");
    }

    printf("            Load Current = ");
    if (ram->adc_iload_f.valid) {
        printf("%5.2lf A\n", ram->adc_iload_f.value);
    } else {
        printf("n/a\n");
    }
}

void tristarPrintRam(int slave, TRISTAR_RAM *ram)
{
int duty_cycle;
static char *mode[] = {"Charge", "Load", "Diversion", "Lighting"};

    printf("            Slave %d mode = ", slave);
    if (ram->control_mode.valid) {
        switch (ram->control_mode.raw) {
          case 0:
          case 1:
          case 2:
          case 3:
            printf("%s\n", mode[ram->control_mode.raw]);
            break;
          default:
            printf("unknown mode %d\n", ram->control_mode.raw);
        }
    } else {
        printf("n/a\n");
    }

       printf("           serial number = ", ram->serialno.string);
    if (ram->serialno.valid) {
        printf("%s\n", ram->serialno.string);
    } else {
        printf("n/a\n");
    }
       printf("DIP switches at power up = ");
    if (ram->dip_switch.valid) {
        PrintSwitch((UINT8) ram->dip_switch.raw); printf("\n");
    } else {
        printf("n/a\n");
    }
    printf("     Time since power up = ");
    if (ram->Hourmeter.valid) {
        printf("%d hours\n", ram->Hourmeter.raw);
    } else {
        printf("n/a\n");
    }

    switch (ram->control_mode.raw) {
      case 0: PrintChrg(ram); break;
      case 1: PrintLoad(ram); break;
      case 2: break;
      case 3: break;
    }
    printf("    Heatsink Temperature = ");
    if (ram->T_hs.valid) {
        printf("%d C\n", ram->T_hs.raw);
    } else {
        printf("n/a\n");
    }

    printf(" Minimum Battery Voltage = ");
    if (ram->min_batt_v.valid) {
        printf("%5.2lf V\n", ram->min_batt_v.value);
    } else {
        printf("n/a\n");
    }

    printf(" Maximum Battery Voltage = ");
    if (ram->max_batt_v.valid) {
        printf("%5.2lf V\n", ram->max_batt_v.value);
    } else {
        printf("n/a\n");
    }

    printf("                  Alarms =");
    if (ram->Alarm.valid) {
        PrintAlarms(ram->Alarm.raw); printf("\n");
    } else {
        printf("n/a\n");
    }

    printf("                  Faults =");
    if (ram->fault.valid) {
        PrintFaults(ram->fault.raw); printf("\n");
    } else {
        printf("n/a\n");
    }

    printf("         Ah (resettable) = ");
    if (ram->Ah_r.valid) {
        printf("%5.2lf %d Ah\n", ram->Ah_r.value, ram->Ah_r.raw);
    } else {
        printf("n/a\n");
    }

    printf("              Ah (total) = ");
    if (ram->Ah_t.valid) {
        printf("%5.2lf %d Ah\n", ram->Ah_t.value, ram->Ah_t.raw);
    } else {
        printf("n/a\n");
    }

    printf("                     Kwh = ");
    if (ram->kwh.valid) {
        printf("%5.2lf %d Ah\n", ram->kwh.value, ram->kwh.raw);
    } else {
        printf("n/a\n");
    }

    printf("       reference voltage = ");
    if (ram->V_ref.valid) {
        printf("%5.2lf %d V\n", ram->V_ref.value, ram->V_ref.raw);
    } else {
        printf("n/a\n");
    }

    printf("          PWM duty cycle = ");
    if (ram->d_filt.valid) {
        printf("%.1lf%%\n", ram->d_filt.value);
    } else {
        printf("n/a\n");
    }

}

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2012 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: print.c,v $
 * Revision 1.2  2012/05/02 18:25:05  dechavez
 * tristarPrintRam() prints "n/a" if register reading is not valid
 *
 * Revision 1.1  2012/04/25 21:04:13  dechavez
 * initial release
 *
 */
