#pragma ident "$Id: read.c,v 1.2 2012/05/02 18:22:04 dechavez Exp $"
/*======================================================================
 *
 *  Read things
 *
 *====================================================================*/
#include "tristar.h"

BOOL tristarReadRegister(modbus_t *ctx, TRISTAR_REG *reg)
{
UINT16 word;

    if (modbus_read_registers(ctx, reg->addr.lo, 1, &word) == -1) return reg->valid = FALSE;
    reg->raw = word;

    if (reg->addr.hi != TRISTAR_ADDR_NONE) {
        if (modbus_read_registers(ctx, reg->addr.hi, 1, &word) == -1) return reg->valid = FALSE;
        reg->raw |= (word << 16);
    }

    reg->value = reg->raw * reg->scaling;
    return reg->valid = TRUE;
}

BOOL tristarReadSerial(modbus_t *ctx, char *dest)
{
char *ptr;
TRISTAR_REG reg = {{TRISTAR_ADDR_NONE, 0}, 0.0, 0, 0.0, FALSE};

    ptr = dest;

    reg.addr.lo = TRISTAR_ADDR_SERIAL01;
    if (!tristarReadRegister(ctx, &reg)) return FALSE;
    *ptr++ = (reg.raw & 0x000000ff);
    *ptr++ = (reg.raw & 0x0000ff00) >> 8;

    reg.addr.lo = TRISTAR_ADDR_SERIAL23;
    if (!tristarReadRegister(ctx, &reg)) return FALSE;
    *ptr++ = (reg.raw & 0x000000ff);
    *ptr++ = (reg.raw & 0x0000ff00) >> 8;

    reg.addr.lo = TRISTAR_ADDR_SERIAL45;
    if (!tristarReadRegister(ctx, &reg)) return FALSE;
    *ptr++ = (reg.raw & 0x000000ff);
    *ptr++ = (reg.raw & 0x0000ff00) >> 8;

    reg.addr.lo = TRISTAR_ADDR_SERIAL67;
    if (!tristarReadRegister(ctx, &reg)) return FALSE;
    *ptr++ = (reg.raw & 0x000000ff);
    *ptr++ = (reg.raw & 0x0000ff00) >> 8;

    *ptr++ = 0;

    return TRUE;
}

void tristarReadRam(modbus_t *ctx, int slave, TRISTAR_RAM *dest)
{
TRISTAR_RAM ram_default = TRISTAR_RAM_DEFAULT;

    modbus_set_slave(ctx, slave);
    *dest = ram_default;

    dest->serialno.valid = tristarReadSerial(  ctx, dest->serialno.string);
    tristarReadRegister(ctx, &dest->adc_vb_f);
    tristarReadRegister(ctx, &dest->adc_vs_f);
    tristarReadRegister(ctx, &dest->adc_vx_f);
    tristarReadRegister(ctx, &dest->adc_ipv_f);
    tristarReadRegister(ctx, &dest->adc_iload_f);
    tristarReadRegister(ctx, &dest->Vb_f);
    tristarReadRegister(ctx, &dest->T_hs);
    tristarReadRegister(ctx, &dest->T_batt);
    tristarReadRegister(ctx, &dest->V_ref);
    tristarReadRegister(ctx, &dest->Ah_r);
    tristarReadRegister(ctx, &dest->Ah_t);
    tristarReadRegister(ctx, &dest->Hourmeter);
    tristarReadRegister(ctx, &dest->Alarm);
    tristarReadRegister(ctx, &dest->fault);
    tristarReadRegister(ctx, &dest->dip_switch);
    tristarReadRegister(ctx, &dest->control_mode);
    tristarReadRegister(ctx, &dest->control_state);
    tristarReadRegister(ctx, &dest->load_state);
    tristarReadRegister(ctx, &dest->d_filt);
    tristarReadRegister(ctx, &dest->kwh);
    tristarReadRegister(ctx, &dest->min_batt_v);
    tristarReadRegister(ctx, &dest->max_batt_v);
}

void tristarReadRamLogset(modbus_t *ctx, int slave, TRISTAR_RAM *dest)
{
TRISTAR_RAM ram_default = TRISTAR_RAM_DEFAULT;

    modbus_set_slave(ctx, slave);
    *dest = ram_default;

    switch (slave) {
      case 1:
        tristarReadRegister(ctx, &dest->control_state);
        tristarReadRegister(ctx, &dest->Alarm);
        tristarReadRegister(ctx, &dest->fault);
        tristarReadRegister(ctx, &dest->adc_vb_f);
        tristarReadRegister(ctx, &dest->adc_ipv_f);
        tristarReadRegister(ctx, &dest->adc_vs_f);
        tristarReadRegister(ctx, &dest->adc_vx_f);
        break;
      case 2:
        tristarReadRegister(ctx, &dest->control_state);
        tristarReadRegister(ctx, &dest->Alarm);
        tristarReadRegister(ctx, &dest->fault);
        tristarReadRegister(ctx, &dest->adc_vx_f);
        tristarReadRegister(ctx, &dest->adc_iload_f);
        break;
    }
}

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2012 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: read.c,v $
 * Revision 1.2  2012/05/02 18:22:04  dechavez
 * set valid field in tristarReadRegister(), changed tristarReadRam() to void
 * now that valid field is used, added tristarReadRamLogset()
 *
 * Revision 1.1  2012/04/25 21:04:13  dechavez
 * initial release
 *
 */
