/*======================================================================
 *
 * Include file of constants, type definitions, function declarations
 * Author: Mario Aranha, Berkeley Seismological Laboratory
 * Copyright (C) 2013 Regents of the University of California.
 *
 *====================================================================*/
#ifndef tee2ms_h_defined
#define tee2ms_h_defined

extern char *VersionIdentString;

#include "util.h"
#include "ida10.h"

#define MAXLINE         512
#define	DEFAULT_BLKSIZE 4096
#define	MAX_BLKSIZE	8192
#define MAXSTR          256
#define MAX_SNCL        26
#define	MAX_OUTDATA	2000000
#define DEF_OUTDATA     30000
#define CHUNK_SIZE      10000
#define	DEBUG_HEADER	1
#define	DEBUG_DATA	2
#define	DEBUG_MAP	4
#define	DEBUG_ANY	(DEBUG_HEADER | DEBUG_DATA | DEBUG_MAP)
#define MAX_HASH	(3*MAX_SNCL)
#define NANOSEC_PER_OFISTIC 10000LL

#define	debug(flag)	((debug_flag & (flag)))

/* channel hash structure  */
struct hash_info {
  /* fixed attributes - these are set once for each channel and do not change */
  char *key;			/* IDA channel name = hash key */
  char sncl[MAXSTR];		/* S.N.C.L mapping */
  DATA_HDR *shdr;		/* SEED header */
  int index;           		/* hash array index */

  /* attributes that change with each new packet */
  IDA10_TSHDR first_ihdr;	/* first IDA10 header for this timeseries */
  IDA10_TSHDR prev_ihdr;	/* previous IDA10 header for this timeseries */
  IDA10_TSHDR curr_ihdr;	/* current IDA10 header for this timeseries */
  long ts_nsamp;		/* # samples for this timeseries */
  long ts_npkt;			/* # packets for this timeseries */
  long chan_nsamp;		/* # samples for this channel */
  long chan_npkt;		/* # packets for this channel */
  int nout; 			/* # of samples accumulated for conversion to mSEED */

  /* timeseries attributes - *may* change with each new packet */
  size_t nalloc;		/* # of data items for which memory allocated */
  SEED_FLOAT av_rate;    	/* average sample rate in samples/sec */
  UINT64 first_offset;	        /* first stable offset (ext - pps) in ns for this channel */
  UINT64 curr_offset;	        /* current stable offset (ext - pps) in ns for this channel */
  BOOL is_reboot;		/* femtometer system reboot in progress */
  BOOL is_questionable_ttag;    /* timestamp(s) for this timeseries is questionable */
  BOOL large_offset;            /* previous offset unexpectedly large */

  /* mSEED data storage */
  struct {
    int *p_int;
    float *p_float;
    double *p_double;    
  } databuf;			/* pointer to buffer for accumulated data */


  /* attributes of file */
  FILE *outf;			/* output file descriptor */
  struct {
    IDA10_TSHDR pkt;
    REAL64 value; /* |IDA_packet_time - nominal_expected_time| */
    long ts_nsamp; /* # samples in timeseries up to this packet */
    BOOL clock_ok;
  } error;
  BOOL is_first_pkt;  /* first packet for this channel in file */
  BOOL is_sequential; /* are all packets in sequence */
  UINT64 first_valid_offset; /* first valid offset in file in nanosecs */
  UINT64 last_valid_offset;  /* last valid offset in file in nanosecs */
};

#endif
